
## Usage


### Example: Pre-process CSS

**Using a plugin**

Use plugins to pre-process CSS (Assemble can run any gulp plugin):

```js
var assemble = require('assemble');
var less = require('gulp-less');

app.task('css', function () {
  app.src('styles/*.less')
    .pipe(less())
    .pipe(app.dest('dist/assets/css'));
});

app.task('default', ['css']);
```

**Or, using an engine**

Instead of a plugin you can register an engine, such as [engine-less](https://github.com/jonschlinkert/engine-less).

_(Engines are run automatically on any files that have a file extension matching the name that you used when registering the engine.)_

```js
var assemble = require('assemble');
app.engine('less', require('engine-less'));

app.task('css', function () {
  app.src('styles/*.less')
    .pipe(app.dest('dist/assets/css'));
});

app.task('default', ['css']);
```


***

# Docs

<br>

## Templates

### .partial

> Add partials to be used in other templates.

```js
app.partial('notice', { content: '<strong>...</strong>' });
app.partial('banner', { content: '/*! Copyright (c) 2014 Jon Schlinkert, Brian Woodward... */' });
// or load a glob of partials
app.partials('partials/*.hbs');

// optionally pass locals, all template types support this
app.partials('partials/*.hbs', {site: {title: 'Code Project'}});
```

**Usage**

Use the `partial` helper to inject into other templates:

```js
{{partial "banner"}}
```

Get a cached partial:

```js
var banner = app.views.partials['banner'];
```

### .page

> Add pages that might be rendered (really, any template is renderable, pages fit the part though)

```js
app.page('toc.hbs', { content: 'Table of Contents...'});
// or load a glob of pages
app.pages('pages/*.hbs', {site: {title: 'Code Project'}});
```

Use the `page` helper to inject pages into other templates:

```js
{{page "toc"}}
```

Get a cached page:

```js
var toc = app.views.pages['toc'];
```

Pages are `renderable` templates, so they also have a `.render()` method:

```js
var toc = app.views.pages['toc'];
// async
toc.render({}, function(err, content) {
  console.log(content);
});

// or sync
var res = toc.render();
```

**Params**

 - `locals` **{Object}**: Optionally pass locals as the first arg
 - `callback` **{Function}**: If a callback is passed, the template will be rendered async, otherwise sync.


### .layout

> Add layouts, which are used to "wrap" other templates:

```js
app.layout('default', {content: [
  '<!DOCTYPE html>',
  '  <html lang="en">',
  '  <head>',
  '    <meta charset="UTF-8">',
  '    <title>{%%= title %}</title>',
  '  </head>',
  '  <body>',
  '    {%% body %}', // `body` is the insertion point for another template
  '  </body>',
  '</html>'
].join('\n')});

// or load a glob of layouts
app.layouts('layouts/*.hbs', {site: {title: 'Code Project'}});
```

Layouts may be use with any other template, including other layouts. Any level of nesting is also possible.

**Body tags**

Layouts use a `body` as the insertion point for other templates. The syntax assemble uses for the `body` tag is:

```js
{%% body %}
```

Admittedly, it's a strange syntax, but that's why we're using it. assemble shouldn't collide with templates that you might be using in your documentation.


**Usage**

Layouts can be defined in template locals:

```js
// either of these work (one object or two)
app.page('toc.hbs', { content: 'Table of Contents...'}, { layout: 'default' });
app.partial('foo.hbs', { content: 'partial stuff', layout: 'block' });
```

Or in the front matter of a template. For example, here is how another layout would use our layout example from earlier:

```js
// using this 'inline' template format to make it easy to see what's happening
// this could be loaded from a file too
app.layout('sidebar', {content: [
  '---',
  'layout: default',
  '---',
  '<div>',
  ' {%% body %}',
  '</div>'
].join('\n')});
```

### .engine

> Add engines for rendering templates.

```js
// render any files with a `.tmpl` extension using engine-lodash
app.engine('tmpl', require('engine-lodash'));

// render any files with a `.less` extension using engine-less
app.engine('less', require('engine-less'));
```


### .helper

> Add helpers to be used in templates.

Helpers are passed to the template engine being used at render time.

**Custom helper**

```js
app.helper('read', function(filepath) {
  return fs.readFileSync(filepath, 'utf8');
});
//=> {{read "foo.txt"}}
```

**Register a glob of helpers**

```js
app.helpers('helpers/*.js');
```

**Pro tip**

If you want to publish your helpers and share them with the community, make them as generic as possible so they work with any template engine.


# Data

### .data

> Load data to pass to templates.

Any of these work:

```js
app.data({foo: 'bar'});
app.data('package.json');
app.data(['foo/*.{json,yml}']);
```

# API
{%= apidocs("index.js") %}

## Run tests
{%= include("tests") %}

## Test coverage

```
{%= coverage("coverage/summary.txt") %}
```

## Contributing
{%= include("contributing") %}

## Authors
{%= include("authors", maintainers) %}

## License
{%= copyright({year: 2014}) %}
Copyright (c) 2014 Fractal <contact@wearefractal.com> (for completions and CLI)
{%= license %}

***

{%= include("footer") %}
